package javax.swing;

import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.swing.plaf.ButtonUI;
import java.beans.ConstructorProperties;
import javax.accessibility.Accessible;

public class JButton extends AbstractButton implements Accessible
{
    private static final String uiClassID = "ButtonUI";
    
    public JButton() {
        this(null, null);
    }
    
    public JButton(Icon icon) {
        this(null, icon);
    }
    
    @ConstructorProperties({ "text" })
    public JButton(String s) {
        this(s, null);
    }
    
    public JButton(Action action) {
        this();
        this.setAction(action);
    }
    
    public JButton(String s, Icon icon) {
        this.setModel(new DefaultButtonModel());
        this.init(s, icon);
    }
    
    @Override
    public void updateUI() {
        this.setUI((ButtonUI)UIManager.getUI(this));
    }
    
    @Override
    public String getUIClassID() {
        return "ButtonUI";
    }
    
    public boolean isDefaultButton() {
        JRootPane rootPane = SwingUtilities.getRootPane(this);
        return rootPane != null && rootPane.getDefaultButton() == this;
    }
    
    public boolean isDefaultCapable() {
        return this.defaultCapable;
    }
    
    public void setDefaultCapable(boolean defaultCapable) {
        this.firePropertyChange("defaultCapable", this.defaultCapable, this.defaultCapable = defaultCapable);
    }
    
    @Override
    public void removeNotify() {
        JRootPane rootPane = SwingUtilities.getRootPane(this);
        if (rootPane != null && rootPane.getDefaultButton() == this) {
            rootPane.setDefaultButton(null);
        }
        super.removeNotify();
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("ButtonUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",defaultCapable=" + (this.defaultCapable ? "true" : "false");
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JButton.AccessibleJButton();
        }
        return this.accessibleContext;
    }
    
    protected class AccessibleJButton extends AbstractButton.AccessibleAbstractButton
    {
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.PUSH_BUTTON;
        }
    }
}
